/* main.cpp - Initializes the base
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

extern "C" void __cxa_pure_virtual();
void __cxa_pure_virtual(void){};

#include "HardwareSerial.h"
#include "shell.h"
#include "timer2.h"
#include <avr/delay.h>
#include "usb_keyboard.h"
#include "shortcut_converter.h"
#include "VirtualWire/VirtualWire.h"


#define RX_LED_PIN PIN5
#define ON_LED_PIN PIN6
#define led_on(pin) PORTD |= _BV(pin)
#define led_off(pin) PORTD &= ~_BV(pin)

#define VW_RX_LED_TIME 1000
#define UART_RX_LED_TIME 500
#define RX_BUFFER_SIZE 20

uint64_t led_timer = 0;
unsigned char incoming;

bool verbosity = false;
unsigned int number_of_packets_received = 0;
uint8_t buffer[RX_BUFFER_SIZE];
uint8_t buffer_length = RX_BUFFER_SIZE;

USBKeyboard keyboard;
ShortcutConverter shortcut_converter;
Shell shell(&shortcut_converter, &keyboard);

int main(void)
{
	Timer2::init();
	DDRD |= _BV(RX_LED_PIN) + _BV(ON_LED_PIN);
	led_on(ON_LED_PIN);
	Serial.begin(9600);
	vw_set_rx_pin(3);
	vw_setup(2000);
	vw_rx_start();

	_delay_ms(200); //Gives a delay to allow a terminal to display the message that follows.

    Serial.println_P(PSTR("keyMote Copyright (C) 2010 Antoine Mercier-Linteau"));
    Serial.println();
    Serial.println_P(PSTR("This program comes with ABSOLUTELY NO WARRANTY!"));
    Serial.println_P(PSTR("This is free software and hardware; you are welcome to"));
    Serial.println_P(PSTR("redistribute it under certain conditions."));
	shell.prompt();

	while(true)
	{
		if(vw_get_message(buffer, &buffer_length))
		{
			if(verbosity)
			{
				Serial.print_P(PSTR("Received packet:"));
				for(uint8_t i = 0; i < buffer_length; i++)
				{
					Serial.print(buffer[i], HEX);
					Serial.print(" ");
				}
				Serial.println("");
			}
			shortcut_converter.take(buffer, buffer_length);
			buffer_length = RX_BUFFER_SIZE; //resets buffer_length
		}


		if(shortcut_converter.available())
		{
			number_of_packets_received++;
			char keys[4];
			shortcut_converter.read(keys);
			led_timer = Timer2::millis() + VW_RX_LED_TIME;
			led_on(RX_LED_PIN);

			keyboard.take_report(keys);
		}

		keyboard.run();

		if (Serial.available())
		{
			incoming = Serial.read();

			shell.take_char(incoming);
			led_on(RX_LED_PIN);
			led_timer = Timer2::millis() + UART_RX_LED_TIME;
		}

		if(Timer2::millis() > led_timer)
		{
			led_off(RX_LED_PIN);
		}
	}

	return 0;
}


