/* shell.h - Implements a serial shell
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SHELL_H_
#define SHELL_H_

#include <stdint.h>
#include "shortcut_converter.h"
#include "usb_keyboard.h"

#define MAX_COMMAND_BUFFER_SIZE 50
#define MIN_COMMAND_BUFFER_SIZE 10
#define STEP_SIZE 10

class Shell
{
	private:
		char* command_buffer;
		uint8_t position;
		uint8_t level;
		uint8_t end;
		bool escape;
		char command_char;
		ShortcutConverter* shortcut_converter;
		USBKeyboard* keyboard;


	public:
		Shell(ShortcutConverter* sc, USBKeyboard* kb);

		void take_char(char incoming);
		void prompt(void);
	private:
		void execute_command(unsigned char argc, char* argv[]);
		void parse_command();

	protected:
		uint8_t do_help(unsigned char argc, char* argv[]);
		uint8_t do_verbosity(unsigned char argc, char* argv[]);
		uint8_t do_info(unsigned char argc, char* argv[]);
		uint8_t do_key(unsigned char argc, char* argv[]);
		uint8_t do_map(unsigned char argc, char* argv[]);
		uint8_t do_sim_type(unsigned char argc, char* argv[]);
		uint8_t do_sim_packet(unsigned char argc, char* argv[]);
		uint8_t do_inspect(unsigned char argc, char* argv[]);

};

#endif /* SHELL_H_ */
