/* shortcut_converter.h - emulates a keyboard
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef KB_EMULATOR_H_
#define KB_EMULATOR_H_

#define SHORTCUT_BUFFER_SIZE 6

#define MAX_MAP_SIZE 0xFE
#define ERROR_ID 0xFF

#include <stdint.h>

typedef struct
{
	uint8_t txid;
	uint8_t button_id;
	uint8_t key0;
	uint8_t key1;
	uint8_t key2;
	uint8_t key3;
} entry;

typedef struct
{
	uint8_t size;
	entry* entries;
} shortcut_map;

class ShortcutConverter
{
	private:
		shortcut_map map;
		uint8_t shortcut_buffer[SHORTCUT_BUFFER_SIZE];
		uint8_t start;
		uint8_t end;

	public:
		ShortcutConverter();

		unsigned char available();
	     /*for now, it can only take one shortcut at a time*/
		int8_t take(uint8_t txid, uint8_t button_id);
		int8_t take(uint8_t buffer[], uint8_t length);
		uint8_t read(char dest[]);
		void load_map();
		shortcut_map* const get_map();
		int8_t save_entry( uint8_t txid,
				uint8_t button_id,
				uint8_t key0,
				uint8_t key1,
				uint8_t key2,
				uint8_t key3);
		int8_t delete_entry(uint8_t txid, uint8_t button_id);
		uint8_t find_entry(uint8_t txid, uint8_t button_id);
		void reset(void);

		static uint8_t inspect(int addr);

	private:
		uint8_t map_to_combination(unsigned short id);

};

#endif /* KB_EMULATOR_H_ */
