/* timer2.cpp - Abstracts timer2 into a class
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "timer2.h"
#include <avr/io.h>


void Timer2::init(void)
{
	TCCR2A |= _BV(WGM21); //Timer clear on output match of OCR2A
	TCCR2B |= _BV(CS22) + _BV(CS21); //Prescaler at 256
	TIMSK2 |= _BV(OCIE2A); //Interrupt on overflow
	//Nothing to set for TIFR2
	//Nothing to set for ASSR

	/*If the internal RC oscillator is used, the pins taken by the external crystal
	could be used for a 32.768kHz oscillator that could run this timer asynchronously*/

	GTCCR |= _BV(PSRASY); //Clears the prescaler register
	TCNT2 = 0; //Clears the timer
	OCR2A = TIMER2_OUTPUT_COMPARE;

	milliseconds = 0;
}

uint64_t Timer2::millis(void)
{
	uint64_t val;
	cli();
	val = milliseconds;
	sei();
	return val;
}

ISR(TIMER2_COMPA_vect, ISR_NOBLOCK)
{
	//Interrupts deactivated by hardware
	milliseconds += MS_TO_TIMER2_OVERFLOW;
	//Interrupts reactivated by compiler
}

