/* kbhid.h - implements a HID compatible USB keyboard
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USB_KEYBOARD_H_
#define USB_KEYBOARD_H_

extern "C"
{
	#include "usb/usbdrv.h"
}

typedef struct
{
	unsigned char modifiers;
	unsigned char key0;
	unsigned char key1;
	unsigned char key2;
} report;

#define REPORT_BUFFER_SIZE 3
#define KEY_TIMEOUT_MS 3000

class USBKeyboard
{

	private:
		unsigned char idleRate;
		uint64_t idleCounter;
		uint64_t key_timeout;

		report report_buffer[REPORT_BUFFER_SIZE];
		unsigned char start;
		unsigned char end;

		report outgoing_report;

		bool depress;

	public:
		USBKeyboard();

		char take_report( char keys[]);
		void build_report(void);
		void run();
		static char* usage_id_to_string( unsigned char id);
		static unsigned char string_to_usage_id( char* str );

		friend usbMsgLen_t usbFunctionSetup(uchar data[8]);

};

static USBKeyboard* instance;

#endif /*  USB_KEYBOARD_H_ */
