/* remote.cpp - main loop and globals
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include <avr/power.h>
#include <avr/sleep.h>
#include "keypad.h"
#include "packetizer.h"
#include "timer0.h"
#include "VirtualWire/VirtualWire.h"

#define led_on() PORTB |= _BV(PORTB2)
#define led_off() PORTB &= ~(_BV(PORTB2))
#define led_test() PINB & _BV(PIN2)

Keypad keypad;
Packetizer packetizer;

uint32_t led_timer = 250;
uint8_t buffer[8];

int main(void)
{

	DDRB |= _BV(DDB2);
	Timer0::init();
	led_on();
	//No need to set tx pin, it has been hard coded in the module.
	vw_setup(2000);
	power_adc_disable();
	power_usi_disable();

	ACSR |= _BV(ACD); //Disables analog comparator

	while(true)
	{
		keypad.scan();

		if(keypad.available_down())
		{
				packetizer.make(keypad.available_down(), (packet*)buffer);
				while(keypad.available_down())
				{
					packetizer.take(keypad.read_down());
				}

				vw_send(buffer, ((packet*)buffer)->len);
				led_timer = Timer0::millis() + 250;
				led_on();
		}

		if(led_timer < Timer0::millis())
		{
			led_off();
		}


		if(vw_tx_active())
		{
			set_sleep_mode(SLEEP_MODE_IDLE); /*A deeper sleep mode cannot be used here
			since we need timer1 for the VirtualWire module and timer0 to wakeup.*/
		}
		else if( keypad.keys_bouncing ||
			keypad.available_down() ||
			led_test())
		{
			set_sleep_mode(SLEEP_MODE_IDLE);

		}
		else
		{
			set_sleep_mode(SLEEP_MODE_PWR_DOWN);
		}
		sei();
		sleep_enable();
		sleep_cpu();
		sleep_disable();
	}

	return 0;
}

