/* timer0.cpp - abstracts timer0 into a class
 * Copyright (C) 2010 Antoine Mercier-Linteau
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "timer0.h"
#include <avr/io.h>


void Timer0::init(void)
{
	TCCR0A |= _BV(WGM01); //Timer clear on output match of OCR1A
	TCCR0B |= _BV(CS02) + _BV(CS00); //Prescaler at 1024
	TIMSK0 |= _BV(OCIE0A); //Interrupt on overflow
	//Nothing to set for TIFR0

	TCNT0 = 0; //Clears the timer
	OCR0A = TIMER0_OUTPUT_COMPARE;

	milliseconds = 0;
}

uint32_t Timer0::millis(void)
{
	uint32_t val;
	cli();
	val = milliseconds;
	sei();
	return val;
}

ISR(TIM0_COMPA_vect)
{
	//Interrupts deactivated by hardware
	milliseconds += MS_TO_TIMER0_OVERFLOW;
	//Interrupts reactivated by compiler

}

